/*
 * IoT Tea Timer - set an alarm on a nanoESP, receive it via inet (WiFi).
 *
 * (C) 2016 General Overnight  <generalovernight.wordpress.com>
 *
 * This is Free Software available under the GNU General Public License (GPL).
 *
 * $Id: IoTeaTimer.ino,v 1.6 2016-01-31 08:55:30 manolo Exp $
 */

/* Include Arduino.h before any local header files as ist provides the
 * standard types used therein (like boolean or String).
 */
#include <Arduino.h>

#include "mytones.h"
#include "logdebug.h"
#include "wlansetup.h"
#include "serialcomm.h"

/* ********************** */
/* *** Debug settings *** */
/* ********************** */

/* NOTE: uncomment the following line for fully silent operation
 * regardless of the DEBUG setting below (production code).
 */
#define NDEBUG

/* We use a subset of the standard syslog levels for classifying debug
 * output here. These are defined in "logdebug.h" and include can be
 * specified with the macros LVL_EROR, LVL_WARN, LVL_NOTE, LVL_INFO and
 * LVL_DBUG.
 */

/* Set the minimum level of debug messages to be printed (defaults to
 * LVL_WARN if undefined)
 */
#define DEBUG LVL_INFO

#ifndef NDEBUG
/* This flag enables short action timeouts so you don't have to wait a
 * long time when testing the corresponding functions.
 */
# define TEST_ACTIONS
/* NOTE: comment out the following line to enable TEST_ACTIONS */
# undef TEST_ACTIONS
#endif

/* *********************************** */
/* *** End section: Debug settings *** */
/* *********************************** */

/* ************************************ */
/* *** Custom Settings - Edit here! *** */
/* ************************************ */

/* Read in the SSID, PSK and IP settings from secrets.h (that file has
 * to be created locally, see example-secrets.h)
 */
#include "secrets.h"

/* Default timeout for AT commands */
#define DEFTO 6000

#ifdef TEST_ACTIONS
/* Reduced length of a minute to speed up testing (this should be
 * greater than ADJTIME + ALRMLEN)
 */
# define MINUTE_LEN 12
#else
# define MINUTE_LEN REAL_MINUTE
#endif

/* Adjust the timeout by this many seconds to account for the time spent
 * sending the alarm over the wireless network.
 */
#define ADJTIME 18

/* The duration of the alarm sound will be subtracted from the timeout
 * so that the end of the sound output marks the precise end of the
 * timeout.
 */
#define ALRMLEN 20

/* Blink the onboard LED D3 (pin 13) to indicate activity if true, else
 * the RGB-LED.
 */
//#define USE_ONBOARD_LED true
#define USE_ONBOARD_LED false

/* Red timer: 5 min */
#define TIME_R 5
/* Green timer 15 min */
#define TIME_G 15

/* ** Destination UDP Port numbers, defined as strings for convenience ** */
/* Red alarm */
#define PORT_R "1070"
/* Green alarm */
#define PORT_Y "1071"
/* Blue alarm */
#define PORT_G "1072"
/* Yellow alarm */
#define PORT_B "1073"

/* Define (a standard) I/O pin setup */
#define BTN0 2 /* Interrupt */
#define BTN1 3 /* Interrupt (PWM also, but tone output may interfere) */
#define LED_COM 8
#define LED_R 5 /* PWM */
#define LED_G 6 /* PWM */
#define LED_B 9 /* PWM */
#define BUZZ 10 /* PWM */

/* *************************** */
/* *** End custom Settings *** */
/* *************************** */

/* number of seconds in a real (wall clock) minute */
#define REAL_MINUTE 60

/* maximum value for PWM outputs */
#define MAX_PWM 255

/* Set up the serial connction between Atmega MCU and ESP WLAN-Module */
#include <SoftwareSerial.h>
SoftwareSerial esp8266(11, 12);

/* *********************************** */
/* *** Custom Functions: Edit here *** */
/* *********************************** */

/* *** Event handler *** */

int handleButton(int btn)
{
  int rc = 0;
  int led = -1;
  int timeout = 0;
  String port = "-";
  unsigned long start = millis();

  switch (btn)
  {
    case BTN0:
      led = LED_R;
      port = PORT_R;
      timeout = TIME_R * MINUTE_LEN - ADJTIME - ALRMLEN;
      break;
    case BTN1:
      led = LED_G;
      port = PORT_G;
      timeout = TIME_G * MINUTE_LEN - ADJTIME - ALRMLEN;
      break;
    default:
      /* software error (cf. sysexits.h) */
      rc = 70;
      dbg(LVL_EROR, "Software error: invalid button ID: " + String(btn));
  }

  if (0 == rc)
  {
    dbg(LVL_INFO, "Button activated, timeout = " + String(timeout));

    digitalWrite(led, HIGH);
    /* Who is connected? */
    dbg(LVL_INFO, runATcmd("+CWLIF", 1));
    /* Spend the time until the scheduled timeout in a loop blinking an LED */
    for (int i = timeout; i-- > 0;)
    {
#if USE_ONBOARD_LED
      digitalWrite(LED_BUILTIN, i % 2);
#else
      rgbWrite(led, (unsigned int)((float)MAX_PWM * (float)i / (float)timeout));
#endif
      delay(999);
    }
    /* Timeout! Send an alarm over WiFi first ... */
    if (configUDP(port))
    {
      tone(BUZZ, TONE_a2, 200);
      delay(200);
      /* Someone connected? */
      dbg(runATcmd("+CWLIF", 1));
      // Alas, ESP chokes on this:
      //sendUDP(DSTIP, port, "ALRM");
      sendUDP("ALRM");
      tone(BUZZ, TONE_c1, 200);
      delay(200);
      runATcmd("+CIPCLOSE", "OK");
      tone(BUZZ, TONE_c1, 200);
      delay(200);
    } else {
      dbg("Error opening UDP connection");
      tone(BUZZ, TONE_dis2, 200);
      delay(200);
    }
    dbg(LVL_NOTE,
	"Network alarm finished after " + String((millis() - start) / 1000)
	+ " seconds.");
    /* sound an alarm */
#if USE_ONBOARD_LED
    digitalWrite(LED_BUILTIN, HIGH);
#endif
    for (int i = 5; i-- > 0;)
    {
      rgbWrite(led, LOW);
      tone(BUZZ, TONE_c1);
      /* tone returns immediately, use delay to determine its duration */
      delay(1000);
      rgbWrite(led, LOW);
      tone(BUZZ, TONE_e1);
      delay(1000);
      rgbWrite(led, LOW);
      tone(BUZZ, TONE_g1);
      delay(1000);
      rgbWrite(led, LOW);
      tone(BUZZ, TONE_ais2, 1000);
      delay(1000);
      noTone(BUZZ);
    }
    dbg(LVL_NOTE,
	"Timer finished after " + String((millis() - start) / 1000)
	+ " seconds.");
    /* ... finally reset the timer LED */
    digitalWrite(led, LOW);
  }
}


/* Lights up the RGB LED alternating between full on and a dimmed mode
 * specified by the brightness parameter.
 *
 * TODO: The colour parameter currently only supports red and green
 * blue. It expects the pin number as its value which is to be changed.
 *
 */
void rgbWrite(unsigned int colour, unsigned int brightness)
{
  /* A variable declared static keeps its value across multiple
   * invocations and is initialized only once, when the function is
   * first run.
   */
  static boolean toggle = false;

#ifndef NDEBUG
  /* Use a standard C prinf function to format the string to be output
   * with Serial.println into a C character array of appropriate length
   * (80 characters default terminal line length).
   */
  char msg[82] = "";
  /* One extra byte at the end for the ASCII-NUL terminating the
   * C-string and an additional one as a safety net. 
   */
  snprintf(msg, sizeof(msg)-2,
	   "rgbWrite(colour=%u, brightness=%u)", colour, brightness);
  Serial.println(msg);
#endif
  /* sanity check the brightness value to guard against programming errors */
  if (brightness > MAX_PWM) { brightness = MAX_PWM; }

  /* Reset all LED outputs first */
  digitalWrite(LED_R, LOW);
  digitalWrite(LED_G, LOW);
  digitalWrite(LED_B, LOW);
  if (toggle)
  {
    /* Light up the LED with the specified colour in full brightness */
    switch (colour) {
      case LED_R:
	digitalWrite(LED_R, HIGH);
	break;
      case LED_G:
	digitalWrite(LED_G, HIGH);
	break;
    }
  }
  else
  {
    /* Light up the LED with the specified colour in reduced brightness */
    switch (colour) {
      case LED_R:
	analogWrite(LED_R, brightness);
	break;
      case LED_G:
	analogWrite(LED_G, brightness);
	break;
    }
  }
  /* toggle the value of toggle for the next run */
  toggle = ! toggle;
}


/* **************************** */
/* *** End custom functions *** */
/* **************************** */


/* === Main ============================================================= */

void setup() {
  int rc = 0;
  Serial.begin(19200);
  esp8266.begin(19200);

  tone(BUZZ, TONE_h2, 500);

  if (0 != (rc = espConfig())) {
    tone(BUZZ, TONE_h2, 1500);
    for (int i = rc; i> 0; i>>=1) {
      /* tone returns immediately so a delay is necessary as
       * long as the beep duration at least (second arg to tone).
       */
      delay(1000);
      tone(BUZZ, TONE_h, 500);
    }
    serialDebug();
  } else {
    tone(BUZZ, TONE_c1, 500);
  }

  /* configure I/O */
  pinMode(LED_R, OUTPUT);
  pinMode(LED_G, OUTPUT);
  pinMode(LED_B, OUTPUT);
  pinMode(BTN0, INPUT_PULLUP);
  pinMode(BTN1, INPUT_PULLUP);

  /* Play a triad (Dreiklang) when ready The tone duration is not
   * specified but limited by the following commands.
   */
  tone(BUZZ, TONE_c1);
  delay(500);
  tone(BUZZ, TONE_dis1);
  delay(500);
  tone(BUZZ, TONE_gis1, 500);
  delay(500);
}


void loop() {
  boolean sw0 = !digitalRead(BTN0), sw1 = !digitalRead(BTN1);
#if DEBUG > 1
  static unsigned long timestamp = millis();
  unsigned long now = millis();
  unsigned long runtime = now - timestamp;
  if (runtime) {
    dbg( 7, "Program Uptime: " + String(now / 1000.0)
	 + " s;  Main loop: previous run "
	 + String(runtime) + " ms ago; Buttons: "
	 + String(sw0) + "," + String(sw1));
  } else {
    dbg(7, "Program Uptime: " + String(now / 1000.0)
	+ " s.  Main loop: first run; Buttons: "
	+ String(sw0) + "," + String(sw1));
  }
  timestamp = now;
#endif

  if (sw0 | sw1)
  {
    if (sw0)
    {
      /* sw1 has been pressed */
      handleButton(BTN0);
    }
    else /* sw1 must have been pressed, else we had not entered this loop */
    {
      handleButton(BTN1);
    }
  } else {
    /* Check the serial Atmel-ESP link for pending data, process it, if
     * any or wait a quarter of a second before the next loop run.
     */
    if (!serialIO()) { delay(250); }
  }
}


/* ====== End Main ====================================================== */

// Local Variables:
// mode: c++
// End:
