/*
 * Functions pertaining to serial I/O between the Atmega MCU and the ESP.
 *
 * (C) 2016 General Overnight  <generalovernight.wordpress.com>
 *
 * This is Free Software available under the GNU General Public License (GPL).
 *
 * $Id: serialcomm.ino,v 1.1 2016-01-31 08:55:30 manolo Exp $
 */

#include "serialcomm.h"


boolean sendUDP(String addr, String port, String data)
{
  boolean rc = 0;

  esp8266.setTimeout(DEFTO);
  rc = runATcmd("+CIPSEND=" + String(data.length() + 2) + "," + addr + "," + port,
		data, "OK");

  return rc;
}


/*
 * Send the specified data via UDP.
 *
 * Returns true (1) on error.
 */
boolean sendUDP(String data)
{
  boolean rc = 0;

  esp8266.setTimeout(DEFTO);
  rc = runATcmd("+CIPSEND=" + String(data.length() + 2), data, "OK");

  return rc;
}


/*
 * Run the specified AT command, returning the response
 */
String runATcmd(String command)
{
  String resp = "", cmd = "AT" + command;
  int start;

  esp8266.println(cmd);
  resp = esp8266.readString();
  dbg(LVL_DBUG, "# " + cmd + " = " + resp);
  while (-1 < (start = resp.indexOf(command))) {
    resp.remove(start, 1 + command.length());
  }

  return(resp);
}


/*
 * Run the specified AT command, returning the response but remove at
 * most stripLevel instances of the command name beforehand.
 */
String runATcmd(String command, int stripLevel)
{
  String resp = "", cmd = "AT" + command;
  int start;

  esp8266.println(cmd);
  resp = esp8266.readString();
  dbg(LVL_DBUG, "# " + cmd + " = " + resp);
  while (stripLevel-- > 0 && -1 < (start = resp.indexOf(command))) {
    resp.remove(start, 1 + command.length());
  }

  return(resp);
}


/*
 * Run specified AT command, waiting for an expected response string.
 *
 * Returns true (1) on error
 */
boolean runATcmd(String command, char expect[])
{
  return(runATcmd(command, "", expect));
}


/*
 * Run specified AT command with optional data to send.
 *
 * Returns true (1) on error
 */
boolean runATcmd(String command, String data, char expect[])
{
  boolean rc = false, found = false, withdata = boolean(data.length());
  String response = "", cmd = "AT" + command;

  esp8266.println(cmd);

  while (!(found || rc)) {
    response = esp8266.readStringUntil('\n');
    if (1 + response.indexOf("ERROR")) {
      rc = true;
    } else if (1 + response.indexOf(withdata ? ">" : expect)) {
      found = true;
    }
    dbg(">>> " + response);
  }

  if (withdata) {
    dbg("... sending data: '" + data + "'");
    esp8266.println(data);
    rc |= !esp8266.findUntil(expect, "ERROR");
  }

  dbg(LVL_DBUG, "# AT+" + cmd + " = " + response);

  if (rc)
  {
    dbg(LVL_EROR, "Error sending command: " + cmd + ": " + response);
  } else {
    dbg(LVL_INFO, "Command successful: " + cmd);
  }
  return(rc);
}


/*
 * Returns:
 *   false: when no data were available,
 *   true: when data were exchanged
 */
boolean serialIO()
{
  boolean rc = false;
  unsigned long time;

  /* Read data from the ESP if available, but don't lock out other
   * program parts and stop once data were read.
   */
  while (!rc && esp8266.available()) {
    Serial.write(esp8266.read());
    rc = true;
  }

  while (!rc && Serial.available()) {
    esp8266.write(Serial.read());
    rc = true;
  }

  return(rc);
}


// Local Variables:
// mode: c++
// End:
